
 // Planet-CNC keyboard controller for Leonardo board
 // this version is setup for 32 buttons

 // Includes

#include "Keyboard.h"

 //========================================================================

 //  Pin name Assignments
                           
 int Button = A0; //set pin 24 (analog 2) to read voltage from voltage divider buttons

 //========================================================================

 // Label setups for Constants 

 int StillPushed; // variable to check if button is still pushed 

 int WhichButton; // variable for analog to digital conversion 

 int Repeat;   // variable used for repeat buttons i.e. jogging buttons

 const int DeBounce = 50; // millisecond variable used as switch DeBounce timer when button is pressed

 const int WaitForRelease = 500; // millisecond delay timer to allow for release of button

 String StuckButton = "x";   // string variable to check for a stuck button. Value is changed in each
                            // button section of code that uses the stuck button feature
                            // only needed if serial functions are enabled

 const int StepDelay = 500;    // delay timer for moving motors one step at a time, adjust to your liking

                            
 //=======================================================================

 // define function of each button 

const int F12 = 205;  // in order to prevent accidental commands from the computer keyboard all of the
                      // commands used in the pendent will use 2 key combinations. The pendent will use
                      // just one button to send both key commands at the same time and
                      // one of the keys will always be F12 
                      // 205 is the keyboard code that the Planet CNC software
                      // recognizes for the F12 key press.

const int Button1 = 97;     // keyboard code for letter "a" 
const int Button2 = 98;     // "b"
const int Button3 = 99;     // "c" 
const int Button4 = 100;    // "d"
const int Button5 = 101;    // "e"
const int Button6 = 102;    // "f"
const int Button7 = 103;    // "g"
const int Button8 = 104;    // "h"
const int Button9 = 105;    // "i"  
const int Button10 = 106;   // "j"
const int Button11 = 107;   // "k"
const int Button12 = 108;   // "l"
const int Button13 = 109;   // "m"
const int Button14 = 110;   // "n"
const int Button15 = 111;   // "o"
const int Button16 = 112;   // "p"
const int Button17 = 113;   //"q"
const int Button18 = 114;   //"r"
const int Button19 = 115;   //"s"
const int Button20 = 116;   //"t"
const int Button21 = 117;   //"u"
const int Button22 = 118;   //"v"
const int Button23 = 119;   //"w"
const int Button24 = 120;   //"x"
const int Button25 = 121;   //"y"
const int Button26 = 122;   //"z"
const int Button27 = 48;    //"0"
const int Button28 = 49;    //"1"
const int Button29 = 50;    //"2"
const int Button30 = 51;    //"3"
const int Button31 = 52;    //"4"
const int Button32 = 53;    //"5"

 //==========================================

const int Value00 = 10;       // sets lower limit for button check to allow for noise
                              // on the analog input. Testing so far shows noise to be only about 3 points
 
 // this section is for "greater than" values for the 
 // button functions. These values are created in the
 // resistor network Excel file and can be copied and 
 // pasted from the text file generated by the macro.
 // the number of values will always be one less than
 // the number of buttons used.

 

const int Value31=1011;
const int Value30=973;
const int Value29=940;
const int Value28=908;
const int Value27=876;
const int Value26=843;
const int Value25=811;
const int Value24=779;
const int Value23=747;
const int Value22=715;
const int Value21=683;
const int Value20=651;
const int Value19=619;
const int Value18=587;
const int Value17=555;
const int Value16=523;
const int Value15=492;
const int Value14=460;
const int Value13=428;
const int Value12=396;
const int Value11=365;
const int Value10=333;
const int Value9=301;
const int Value8=269;
const int Value7=238;
const int Value6=206;
const int Value5=174;
const int Value4=142;
const int Value3=110;
const int Value2=79;
const int Value1=47;


 //=====================================================================

 void setup() {
  // put your setup code here, to run once:

  
    pinMode(2,INPUT);       // input for a DMS "Dead man switch"
    digitalWrite(2,HIGH);   // Set pin to high and connect a normally open switch to ground
    
    pinMode(13, OUTPUT);    // Optional >>> to use pin 13 as output for LED to indicate if button is pushed
    
    // turn on internal pullups on unused pins so they are not floating
    pinMode(19,INPUT_PULLUP); 
    //pinMode(20,INPUT_PULLUP); // used for analog input above
    pinMode(21,INPUT_PULLUP);
    pinMode(22,INPUT_PULLUP);
    pinMode(23,INPUT_PULLUP);
    pinMode(24,INPUT_PULLUP);
    pinMode(25,INPUT_PULLUP);
    pinMode(26,INPUT_PULLUP);
    
    analogReference(EXTERNAL); // setup reference voltage function to use an external source. This project 
                               // pulls the USB board supply voltage from the 5 VDC pin and supplies it to 
                               // the top of the voltage divider and to the Analog Reference pin on board

}
 
 //=================================================================================

 void loop() {
  // put your main code here, to run repeatedly:

  Serial.begin(9600); //output to serial monitor for testing of program

//  /*
//   * the serial function is only needed for testing and troubleshooting
//   * Any line that starts with "Serial" can be commented out since 
//   * it is not needed for the program to perform its designated task.
//   * The easiest way to comment out all of the serial commands at once is to go to
//   * the edit feature and run a find and replace. Set find to "Serial." and set 
//   * replace to "//Serial." and that will comment out all the serial lines but still
//   * allow you to re-enable them later by just reversing the "find" and "replace with".
//   */

 WhichButton = analogRead(Button); // read analog input value and convert to digital value.
                                  // Here it is set to read input one time before the start of program loop
                                  // due to random values on first reading of the DAC input on startup per
                                  // Arduino website and testing that showed this to be accurate.
                                  
 //===================================================================================

 Start:

   while(digitalRead(2));   // "Dead man Switch" - Code below will only run if switch is active
                              // This is to prevent accidental button presses on the pendant
                              // from causing unintended machine movement or software action.

  digitalWrite(13,LOW); //Optional > Turns off LED on pin 13 that is used to indicate a button has been pushed.
                        // LED is turned on in the Button1 section and stays on until code loops back to here.

 WhichButton = 0; // set initial value of variable to zero
 WhichButton = analogRead(Button); //read analog input value and convert to digital value

 Serial.println(WhichButton); // send value of analog to digital conversion to serial monitor window

 
 if 
  (WhichButton > Value00); // check to see if any button is pushed. Value00 sets lower limit to allow
                           // some noise on input. Testing so far shows noise to be only about 3 points
                           
  delay(DeBounce); // timer to debounce switch
  
  WhichButton = analogRead(Button); //read button again to get voltage reading of button after debouncing


       String string4 = " start value"; // string used for showing starting value of WhichButton
                                        // variable in serial monitor window
                                        
       String string5 = WhichButton + string4; // combines value of WhichButton with string above
       
       Serial.println(string5);   // prints string in serial monitor to show current value in serial monitor
  

  if(WhichButton > Value00) goto Button1; // if voltage reading is valid, goto correct button
    goto Start;                           // else go back to start and check button again


 //========================================================================================

 IsButtonStillPushed:

            //if button is being held down this Loop
            //prevents the command from being
            //repeated. Button has to be released
            // to go back to start of program.
            // only used on buttons 17 to 32
  
      delay(WaitForRelease);       // timer to allow for releasing button. Adjust value to your
                                   // liking in constants section above
                                   
      StillPushed = analogRead(Button); //get voltage reading of button

        if 
        (StillPushed < Value00) goto Start;  // check digital value to see if button has been released
        
        else
{
        String string2 = " is stuck and analog to digital value is "; // A message that shows in the serial monitor window
        String string3 = StuckButton + string2 + StillPushed;         // if serial function is turned on. These lines                                                                  
        Serial.println(string3);                                      // can be commented out if serial function is                                 
                                                                      // not being used.    

        goto IsButtonStillPushed;     // go back to start of loop to recheck if button has been released.
}
 //=======================================================================================

 // repeat buttons 1 through 16

 // the buttons in this section are setup to repeat sending the character to the keyboard if the button
 // is held down. These buttons will be used for jogging the X, Y and Z axis.
 // the "StuckButton" variable is not used here because these buttons are designed to repeat.
 // The Stuck Button feature is used on buttons 17 and higher

 // Buttons 1 through 6 use a different repeat structure than buttons 7 through 16  
 // to allow single stepping the motor while holding the button down and having enough
 // time between steps to release the button for easier postioning of the machine.
 // Buttons 7 through 16 move the machine continuously while button is held down.

 // comments are only used on the first button of each section since the remaining buttons 
 // in that section are coded the exact same way.

 //=======================================================================================
  Button1:

  digitalWrite(13, HIGH);             // Optional - used to Turn on a LED to show button has been pushed. 
                                      // LED is turned off when program returns to the start loop
  
    if(WhichButton > Value1) goto Button2;    // if Value01 is more than the maximum for this function goto next button else do this function
            

                                          
      Keyboard.begin();                   // turn on keyboard function
      Keyboard.press(F12);                // Equivalent to press and hold key F12
      Keyboard.press(Button1);            // Equivalent to press and hold key assigned to variable "button1"
      Keyboard.releaseAll();              // Release all key functions that are active
      Keyboard.end();                     // turn off keyboard function       
      delay(StepDelay);                   // delay timer so you can hold down the button for a "single step" to move motors one
                                          // step at a time and still have enough time to release button before next step is commanded
                                          
      Serial.print(" button 1 ");         // text output to the Arduino serial monitor screen for testing
      Serial.println(WhichButton);        // shows current value of analog to digital conversion
                                          // values are NOT passed to keyboard. This section was used for testing and troubleshooting
                                          // and is not necessary for program to function so it can be commented out if you want.
                                          
      Repeat = analogRead(Button);        // check to see if button is still being held      

      if(Repeat<Value1 && Repeat>Value00) goto Button1;   // if button is still being held repeat button code to move motor another step
                                                          // else go back to start.

      goto Start;                         // goto to start of program and monitor buttons
 //=========================================================================================
  Button2:
    if(WhichButton > Value2) goto Button3;
      Keyboard.begin();                   
      Keyboard.press(F12);                 
      Keyboard.press(Button2);           
      Keyboard.releaseAll();
      Keyboard.end();             
      delay(StepDelay);
      Serial.print(" button 2 ");   
      Serial.println(WhichButton);         
      Repeat = analogRead(Button);           
      if(Repeat<Value2 && Repeat>Value1) goto Button2;   
      goto Start;                       
 //========================================================================================     
  Button3:
    if(WhichButton > Value3) goto Button4;
      Keyboard.begin();                   
      Keyboard.press(F12);                 
      Keyboard.press(Button3);           
      Keyboard.releaseAll();
      Keyboard.end();              
      delay(StepDelay);
      Serial.print(" button 3 ");   
      Serial.println(WhichButton);                         
      Repeat = analogRead(Button);           
      if(Repeat<Value3 && Repeat>Value2) goto Button3;   
      goto Start;
 //=======================================================================================
  Button4:
    if(WhichButton > Value4) goto Button5;             
      Keyboard.begin();                   
      Keyboard.press(F12) ;                
      Keyboard.press(Button4);           
      Keyboard.releaseAll();
      Keyboard.end();             
      delay(StepDelay);
      Serial.print(" button 4 ");   
      Serial.println(WhichButton);                    
      Repeat = analogRead(Button);
      if(Repeat<Value4 && Repeat>Value3) goto Button4;   
      goto Start;
 //======================================================================================
  Button5:
    if(WhichButton > Value5) goto Button6;
      Keyboard.begin();                   
      Keyboard.press(F12);                 
      Keyboard.press(Button5);
      Keyboard.releaseAll();
      Keyboard.end();
      delay(StepDelay);
      Serial.print(" button 5 ");
      Serial.println(WhichButton);
      Repeat = analogRead(Button);
      if(Repeat<Value5 && Repeat>Value4) goto Button5;       
      goto Start;
 //======================================================================================
  Button6:
    if
      (WhichButton > Value6) goto Button7;
      
      Keyboard.begin();                   
      Keyboard.press(F12) ;                
      Keyboard.press(Button6);
      Keyboard.releaseAll();                           
      Keyboard.end();
      delay(StepDelay);
      Serial.print(" button 6 ");   
      Serial.println(WhichButton);
      Repeat = analogRead(Button);
      if(Repeat<Value6 && Repeat>Value5) goto Button6;
      goto Start;
 //===================================================================================
  Button7:
    if(WhichButton > Value7) goto Button8;

               
      Keyboard.begin();                   // turn on keyboard function
      Keyboard.press(F12);                // Equivalent to press and hold key F12 
      Keyboard.press(Button7);           // Equivalent to press and hold key 
                                          // assigned to variable "button7"
 Loop7:
      delay(DeBounce);                                  // delay timer to allow for debouncing 
                                                        // the release of the button
      
       Repeat = analogRead(Button);                     // get analog value to check if button is still being held down

      Serial.print(" button 7 ");   
      Serial.println(Repeat);
       
      if(Repeat<Value7 && Repeat>Value6) goto Loop7;    // Check to see if button has been released.
                                                        // If button is still being held then repeat button
                                                        // check Loop. When button is released, continue code.
                                                        // Machine will move continuously as long as 
                                                        // button is held down.
                                                        
      Keyboard.releaseAll();                           // Release all key functions that are active
      
      Keyboard.end();                                   // Turn off Keyboard function
      
      goto Start;                                       // return to program start and monitor buttons
 //===================================================================================
  Button8:
    if(WhichButton > Value8) goto Button9;
     
      Keyboard.begin();                   
      Keyboard.press(F12);                 
      Keyboard.press(Button8);
  Loop8:
      delay(DeBounce);
      Repeat = analogRead(Button);
      Serial.print(" button 8 ");   
      Serial.println(Repeat); 
      if(Repeat<Value8 && Repeat>Value7) goto Loop8;
      Keyboard.releaseAll();                           
      Keyboard.end();
      goto Start;
 //===================================================================================
  Button9:
    if(WhichButton > Value9) goto Button10;
               
      Keyboard.begin();                   
      Keyboard.press(F12);                 
      Keyboard.press(Button9);
 Loop9:
      delay(DeBounce);
      Repeat = analogRead(Button);
      Serial.print(" button 9 ");   
      Serial.println(Repeat);
      if(Repeat<Value9 && Repeat>Value8) goto Loop9;
      Keyboard.releaseAll();                           
      Keyboard.end();
      goto Start;
  //===================================================================================
  Button10:
    if(WhichButton > Value10) goto Button11;           
      Keyboard.begin();                   
      Keyboard.press(F12);                 
      Keyboard.press(Button10);
  Loop10:
      delay(DeBounce);
      Repeat = analogRead(Button);
      Serial.print(" button 10 ");   
      Serial.println(Repeat); 
      if(Repeat<Value10 && Repeat>Value9) goto Loop10;
      Keyboard.releaseAll();                           
      Keyboard.end();
      goto Start;
  //===================================================================================
  Button11:
    if(WhichButton > Value11) goto Button12;      
      Keyboard.begin();                   
      Keyboard.press(F12);                 
      Keyboard.press(Button11);
  Loop11:
      delay(DeBounce);
      Repeat = analogRead(Button);
      Serial.print(" button 11 ");   
      Serial.println(Repeat);         
      if(Repeat<Value11 && Repeat>Value10) goto Loop11;
      Keyboard.releaseAll();                           
      Keyboard.end();
      goto Start;
  //===================================================================================
  Button12:
    if(WhichButton > Value12) goto Button13;               
      Keyboard.begin();                   
      Keyboard.press(F12);                 
      Keyboard.press(Button12);
  Loop12:
      delay(DeBounce);
      Repeat = analogRead(Button);
      Serial.print(" button 12 ");   
      Serial.println(WhichButton);
      if(Repeat<Value12 && Repeat>Value11) goto Loop12;
      Keyboard.releaseAll();                           
      Keyboard.end();
      goto Start;
  //===================================================================================
  Button13:
    if(WhichButton > Value13) goto Button14;               
      Keyboard.begin();                   
      Keyboard.press(F12);                 
      Keyboard.press(Button13);
  Loop13:
      delay(DeBounce);
      Repeat = analogRead(Button);
      Serial.print(" button 13 ");   
      Serial.println(Repeat);
      if(Repeat<Value13 && Repeat>Value12) goto Loop13;
      Keyboard.releaseAll();                           
      Keyboard.end();
      goto Start;
  //===================================================================================
  Button14:
    if(WhichButton > Value14) goto Button15;
      Keyboard.begin();                   
      Keyboard.press(F12);                 
      Keyboard.press(Button14);
  Loop14:
      delay(DeBounce);
      Repeat = analogRead(Button);
      Serial.print(" button 14 ");   
      Serial.println(Repeat);
      if(Repeat<Value14 && Repeat>Value13) goto Loop14;
      Keyboard.releaseAll();                           
      Keyboard.end();
      goto Start;
  //===================================================================================
  Button15:
    if(WhichButton > Value15) goto Button16;
      Keyboard.begin();                   
      Keyboard.press(F12);                 
      Keyboard.press(Button15);
  Loop15:
      delay(DeBounce);
      Repeat = analogRead(Button);
      Serial.print(" button 15 ");   
      Serial.println(Repeat);
      if(Repeat<Value15 && Repeat>Value14) goto Loop15;
      Keyboard.releaseAll();                           
      Keyboard.end();
      goto Start;
  //===================================================================================

 Button16:
    if(WhichButton > Value16) goto Button17;
      Keyboard.begin();                   
      Keyboard.press(F12);                 
      Keyboard.press(Button16);
 Loop16:
      delay(DeBounce);
      Repeat = analogRead(Button);
      Serial.print(" button 16 ");   
      Serial.println(Repeat);
      if(Repeat<Value16 && Repeat>Value15) goto Loop16;
      Keyboard.releaseAll();                           
      Keyboard.end();
      goto Start;


 //===================================================================================

 //  code below is For Later Development

 //===================================================================================

 Button17:


    if(WhichButton > Value17) goto Button18; //if value is more than the maximum for this function goto next button else do this function
            
      Serial.print("button 17 ");         //output to serial monintor screen for testing
      Serial.println(WhichButton);         // shows current value of analog to digital conversion
                                          // values are NOT passed to keyboard. This section was used for testing and troubleshooting
                                          // and is not necessary for program to function so it can be commented out if you want.

     StuckButton = " Button 17";          // Variable used for detecting if a button is stuck. Value is changed in each button section of code.
                                          // message is sent to serial monitor only, this line can be commented out if serial function is not used.

      Keyboard.begin();                   // turn on keyboard function
      Keyboard.press(F12);                // Equivalent to press and hold key F12
      Keyboard.press(Button17);           // Equivalent to press and hold key assigned to variable "button17"
      Keyboard.releaseAll();              // Release all key functions that are active
      Keyboard.end();                     // turn off keyboard function      
     
       
      goto IsButtonStillPushed;           // goto  Loop to test if button is being held

 
 //=========================
  Button18:
    if(WhichButton > Value18) goto Button19;
      Serial.println("button 18 ");
      Serial.println(WhichButton);
      StuckButton = " Button 18";
      Keyboard.begin();
      Keyboard.press(F12);
      Keyboard.press(Button18);
      Keyboard.releaseAll();
      Keyboard.end();
      goto IsButtonStillPushed;
 //=========================      
  Button19:
    if(WhichButton > Value19) goto Button20;
      Serial.println("button 19 ");
      Serial.println(WhichButton);
      StuckButton = " Button 19";
      Keyboard.begin();
      Keyboard.press(F12);
      Keyboard.press(Button19);
      Keyboard.releaseAll();
      Keyboard.end();      
      goto IsButtonStillPushed;
 //=========================
  Button20:
    if(WhichButton > Value20) goto Button21;
      Serial.println("button 20 ");
      Serial.println(WhichButton);
      StuckButton = " Button 20";
      Keyboard.begin();
      Keyboard.press(F12);
      Keyboard.press(Button20);
      Keyboard.releaseAll();
      Keyboard.end();        
      goto IsButtonStillPushed;
 //=========================
  Button21:
    if(WhichButton > Value21) goto Button22;
      Serial.println("button 21 ");
      Serial.println(WhichButton);
      StuckButton = " Button 21";
      Keyboard.begin();
      Keyboard.press(F12);
      Keyboard.press(Button21);
      Keyboard.releaseAll();
      Keyboard.end();      
      goto IsButtonStillPushed;
 //=========================
  Button22:
    if(WhichButton > Value22) goto Button23;
      Serial.println("button 22 ");
      Serial.println(WhichButton);
      StuckButton = " Button 22";
      Keyboard.begin();
      Keyboard.press(F12);
      Keyboard.press(Button22);
      Keyboard.releaseAll();
      Keyboard.end();        
      goto IsButtonStillPushed;
 //=========================
  Button23:
    if(WhichButton > Value23) goto Button24;
      Serial.println("button 23 ");
      Serial.println(WhichButton);
      StuckButton = " Button 23";
      Keyboard.begin();
      Keyboard.press(F12);
      Keyboard.press(Button23);
      Keyboard.releaseAll();
      Keyboard.end();        
      goto IsButtonStillPushed;
 //=========================
  Button24:
    if(WhichButton > Value24) goto Button25;
      Serial.println("button 24 ");
      Serial.println(WhichButton);
      StuckButton = " Button 24";
      Keyboard.begin();
      Keyboard.press(F12);
      Keyboard.press(Button24);
      Keyboard.releaseAll();
      Keyboard.end();        
      goto IsButtonStillPushed;
 //=========================
  Button25:
    if(WhichButton > Value25) goto Button26;
      Serial.println("button 25 ");
      Serial.println(WhichButton);
      StuckButton = " Button 25";
      Keyboard.begin();
      Keyboard.press(F12);
      Keyboard.press(Button25);
      Keyboard.releaseAll();
      Keyboard.end();        
      goto IsButtonStillPushed;
 //=========================
  Button26:
    if(WhichButton > Value26) goto Button27;
      Serial.println("button 26 ");
      Serial.println(WhichButton);
      StuckButton = " Button 26";
      Keyboard.begin();
      Keyboard.press(F12);
      Keyboard.press(Button26);
      Keyboard.releaseAll();
      Keyboard.end();        
      goto IsButtonStillPushed;
 //=========================
  Button27:
    if(WhichButton > Value27) goto Button28;
      Serial.println("button 27 ");
      Serial.println(WhichButton);
      StuckButton = " Button 27";
      Keyboard.begin();
      Keyboard.press(F12);
      Keyboard.press(Button27);
      Keyboard.releaseAll();
      Keyboard.end();        
      goto IsButtonStillPushed;
 //=========================
  Button28:
    if(WhichButton > Value28) goto Button29;
      Serial.println("button 28 ");
      Serial.println(WhichButton);
      StuckButton = " Button 28";
      Keyboard.begin();
      Keyboard.press(F12);
      Keyboard.press(Button28);
      Keyboard.releaseAll();
      Keyboard.end();        
      goto IsButtonStillPushed;
 //==================================
 Button29:
    if(WhichButton > Value29) goto Button30;
      Serial.println("button 29 ");
      Serial.println(WhichButton);
      StuckButton = " Button 29";
      Keyboard.begin();
      Keyboard.press(F12);
      Keyboard.press(Button29);
      Keyboard.releaseAll();
      Keyboard.end();        
      goto IsButtonStillPushed;
 //=========================
  Button30:
    if(WhichButton > Value30) goto Button31;
      Serial.println("button 30 ");
      Serial.println(WhichButton);
      StuckButton = " Button 30";
      Keyboard.begin();
      Keyboard.press(F12);
      Keyboard.press(Button30);
      Keyboard.releaseAll();
      Keyboard.end();        
      goto IsButtonStillPushed;
 //=========================
  Button31:
    if(WhichButton > Value31) goto Button32;
      Serial.println("button 31 ");
      Serial.println(WhichButton);
      StuckButton = " Button 31";
      Keyboard.begin();
      Keyboard.press(F12);
      Keyboard.press(Button31);
      Keyboard.releaseAll();
      Keyboard.end();        
      goto IsButtonStillPushed;
 //=========================
  Button32:
    // if(WhichButton > Value32) goto Button33; // This line is not needed since this is
                                                // the last button and it would have the highest value
      Serial.println("button 32 ");
      Serial.println(WhichButton);
      StuckButton = " Button 32";
      Keyboard.begin();
      Keyboard.press(F12);
      Keyboard.press(Button32);
      Keyboard.releaseAll();
      Keyboard.end();        
      goto IsButtonStillPushed;
 //=========================

} // end of program
